package Devel::DLMProf;

use strict;
use warnings;
our $VERSION = '0.04';

1;
__END__

=head1 NAME

Devel::DLMProf -

=head1 SYNOPSIS

  use Devel::DLMProf;

=head1 DESCRIPTION

Devel::DLMProf is dynamic loaded modules detector.

=head1 AUTHOR

Takatoshi Kitano E<lt>kitano.tk@gmail.comE<gt>

=head1 SEE ALSO

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
